c---------------------------------------------------------------------
C!F77-INC
c
c!Description:
c Main data include file for modis processing of the Direct Broadcast
c  Version of the MODIS Sea Surface Temperature product (MODIS product 
c  MOD28).
c
c!Input parameters: None.
c
c!Output Parameters: None.
c
c!Revision History:
c 
c Revision 01.00  2003/07/09
c Written by James E. Davies
c Original version.
c
c!Team-unique Header:
c
c!References and Credits:
c
c
c!END
c---------------------------------------------------------------------

c-----------------------------------------------------------------------
c ... Global Parameters
c-----------------------------------------------------------------------

c ... Maximum character size of input file name
      integer PATH_MAX
      parameter (PATH_MAX = 255)

c ... Number of bands in Direct Broadcast Output file 
c (presently 5 radiances, 5 brightness T, 1 day/night flag, 2 SSTs)

      integer OUT_BAND
      parameter (OUT_BAND = 12)

c ... Maximum number of pixels and lines in a 1km scan

      integer max_pixel, MAX_LINE
      parameter( max_pixel = 1354, MAX_LINE = 10 )

c ... Total number of bands in input file
      integer INBAND
      parameter( INBAND = 36 )

c ... Number of 1km IR bands

      integer nbands
      parameter( nbands = 18 )

c ... Number of 1km IR bands to use as output
      integer n_output 
      parameter ( n_output = 5 )

c ... Bad data value

      real bad_value
      parameter( bad_value = -999.0 )

c ... Number of product QA bytes 

      integer nproduct_qa
      parameter( nproduct_qa = 10 )

     
C ... Number of bands used for sst
      integer sst_band
      parameter ( sst_band = 5)

c-----------------------------------------------------------------------
c ... REAL global variables stored in COMMON
c-----------------------------------------------------------------------

c ... MODIS L1B input radiance data for all 1km IR bands

      real radiance1( max_pixel, max_line, nbands )

c ... Reynolds optimally interpolated weekly global SST

      real oisst(360,180)

c ... MODIS geolocation input latitude and longitude data for 1km bands

      real lat1( max_pixel, max_line ),
     &     lon1( max_pixel, max_line ),
     &     view( max_pixel, max_line ),
     &     solz( max_pixel, max_line ),
     &     sena( max_pixel, max_line ),
     &     sola( max_pixel, max_line )
     


c ### MOD28 processing option

      integer mod28_proc_opt
      parameter ( mod28_proc_opt = 1 )


c ... common output products at full resolution

      real brightness_temp( max_pixel, max_line, n_output ),
     &  raw_radiance( max_pixel, max_line, n_output ),
     &  sst1(max_pixel, max_line), sst4(max_pixel, max_line)
      
c-----------------------------------------------------------------------
c ... INTEGER global variables stored in COMMON
c-----------------------------------------------------------------------

c ... 1km IR band numbers

      integer bands( nbands )      

      integer sstband(sst_band)
      integer sstbandptr(sst_band)
      
c-----------------------------------------------------------------------
c ... BYTE global variables stored in COMMON
c-----------------------------------------------------------------------

c ... Land/water mask for 1km bands

      byte land1( max_pixel, max_line )

c-----------------------------------------------------------------------
c ### COMMON block to hold global variables
c ### (Arrays are listed in the order REALs, INTEGERs, BYTEs)
c-----------------------------------------------------------------------

      common / mod28_data /
     &  radiance1,lat1,lon1,view,solz,sena,sola,raw_radiance,
     &  brightness_temp,bands,land1,oisst,sst1,sst4,sstband,
     &  sstbandptr
