C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine db_mod28_file_open( 
     &                                filename,
     &                                l1b_1km_neles,
     &                                l1b_1km_lun,
     &                                geo_1km_lun,
     &                                mod28_lun,
     &                                hdr_lun,
     &                                debug_lun
     &                              )
      implicit none
      save

      include 'sst.inc'
      include 'db_mod28uw_data.inc'

C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
c Program which will open files and return unit numbers for 
c all files used in the main DB Cloud Top Properties
c and cloud phase program. 
C
C!INPUT PARAMETERS:    None.
C
C!OUTPUT PARAMETERS:
C
C   filename           Configuration file name
C   l1b_1km_neles      Number of elements per scan line of data
C   l1b_1km_lun        LUN for open 1km L1B data file
C   geo_1km_lun        LUN for open geo file
C   mod28_lun          LUN for MOD28 output file
C   hdr_lun            LUN for MOD28 output header file
C   debug_lun          LUN for open debug messages file
C
C!REVISION HISTORY:
c
C!TEAM-UNIQUE HEADER:
C
C!REFERENCES AND CREDITS
C
C EXTERNALS:
C
C       NAMED CONSTANTS:
C
C       FUNCTIONS AND SUBROUTINES:
C               hdrgetkeystr.f
c               db_message.f
C
C!END-------------------------------------------------------------------

c --- constants
      character*18 FUNCTION
      parameter (FUNCTION = 'db_mod28_file_open')

c --- arguments
      integer l1b_1km_neles, l1b_1km_lun, geo_1km_lun, debug_lun, 
     &mod28_lun, hdr_lun
      character*(*) filename

c --- external functions
      integer hdrgetkeystr
      real sbflip
      logical big_endian

c --- internal variables
      character*12 req
      character*80 keyname
      character*(PATH_MAX) file
      integer len_file, keyindex, status, io_err, level, len, i, j
      integer iyrst,imst,idst,iyrnd,imnd,idnd,ndays,index,temp1,temp2
      integer oisst_lun
      real hold
      logical remove_all

c --- Common block used for writing debug output statements
      integer debug
      integer h_output
      common / bug / debug, h_output

c --- initialize the lun
      l1b_1km_lun  = -1
      geo_1km_lun  = -1
      debug_lun	= -1
      hdr_lun	= -1
      oisst_lun = -1

      remove_all = .true.

c --- initialize dummy variables
      req = ' '
      file = ' '


C 1KM Level 1B MODIS DATA FILES

c --- get the file requirement
      keyname = 'L1B_1KM_REQ'
      keyindex = 1
      status = hdrgetkeystr( filename, keyname, keyindex, req )
      if( status.lt.0 ) req = 'OPTIONAL'
      level = 3
      if( req(1:3).eq.'MAN' ) level = 2

c --- get file access information 
      keyname = 'L1B_1KM_NAME'
      keyindex = 1
      status = hdrgetkeystr( filename, keyname, keyindex, file )

c --- check for error reading info file
      if( status.lt.0 ) then 
         call message( 
     &                 FUNCTION,
     &                'FAILED: Read of 1km file info . ',
     &                 0, level 
     &               )

      else
                   
c ------ issue the OPEN request
         OPEN( 
     &         FILE=file,
     &         UNIT=L1B_1KM_UNIT,
     &         FORM=L1B_1KM_FORM,
     &         ACCESS=L1B_1KM_ACCESS,
     &         STATUS=L1B_1KM_STATUS,
     &         RECL=l1b_1km_neles*4,
     &         IOSTAT=io_err, 
     &         ERR=9005
     &       ) 

         call message(
     &                 '--',
     &                'Using 1km L1B file: '// file ,
     &                 0, 4
     &               )

         l1b_1km_lun = L1B_1KM_UNIT
         goto 140

c ------ Failed to open 
9005     continue 
         call message( 
     &                 FUNCTION,
     &                'FAILED: Could not open 1km L1B file. ', 
     &                 0, level 
     &               )

      endif
140   continue
      if( debug.ne.0 ) then
         call strcompress( file, remove_all, len_file )
         write(*,*) '  L1B 1km file: ',file(1:len_file)
      endif
              


C GEOLOCATION FILES: LATITUDE, LONGITUDE, HEIGHT, ...

c --- get the file requirement
      keyname = 'GEO_1KM_REQ'
      keyindex = 1
      status = hdrgetkeystr( filename, keyname, keyindex, req )
      if( status.lt.0 ) req = 'OPTIONAL'
      level = 3
      if( req(1:3).eq.'MAN' ) level = 2

c --- get file access information 
      keyname = 'GEO_1KM_NAME'
      keyindex = 1
      status = hdrgetkeystr( filename, keyname, keyindex, file )

c --- check for error reading info file
      if( status.lt.0 ) then 
         call message( 
     &                 FUNCTION,
     &                'FAILED: Read of Geo file info . ',
     &                 0, level 
     &               )

      else
                   
c ------ issue the OPEN request
         OPEN( 
     &         FILE=file,
     &         UNIT=GEO_1KM_UNIT,
     &         STATUS=GEO_1KM_STATUS,
     &         FORM=GEO_1KM_FORM,
     &         ACCESS=GEO_1KM_ACCESS,
     &         RECL=l1b_1km_neles*4,
     &         IOSTAT=io_err, 
     &         ERR=9006
     &       ) 

         call message(
     &                 '--',
     &                'Using 1km GEO file: '// file ,
     &                 0, 4
     &               )

         geo_1km_lun = GEO_1KM_UNIT
         goto 150

c ------ Failed to open 
9006     continue 
         call message( 
     &                 FUNCTION,
     &                'FAILED: Could not open GEO file. ',
     &                 0, level 
     &               )

      endif
150   continue
      if( debug.ne.0 ) then
         call strcompress( file, remove_all, len_file )
         write(*,*) '  GEO file: ',file(1:len_file)
      endif
            
C OISST FILE

      do i = 1,360
        do j = 1,180
          oisst(i,j) = -1.0
        end do
      end do
c --- get the file requirement
      keyname = 'OISST_REQ'
      keyindex = 1
      status = hdrgetkeystr( filename, keyname, keyindex, req )
      if( status.lt.0 ) req = 'OPTIONAL'
      level = 3
      if( req(1:3).eq.'MAN' ) level = 2

c --- get file access information 
      keyname = 'OISST_NAME'
      keyindex = 1
      status = hdrgetkeystr( filename, keyname, keyindex, file )

c --- check for error reading info file
      if( status.lt.0 ) then 
         call message( 
     &                 FUNCTION,
     &                'FAILED: Read of OISST file info . ',
     &                 0, level 
     &               )

      else

c ------ issue the OPEN request
         OPEN( 
     &         FILE=file,
     &         UNIT=OISST_UNIT,
     &         STATUS=OISST_STATUS,
     &         FORM=OISST_FORM,
     &         ACCESS=OISST_ACCESS,
     &         IOSTAT=io_err, 
     &         ERR=9009
     &       ) 

         call message(
     &                 '--',
     &                'Using OISST file: '// file ,
     &                 0, 4
     &               )

         oisst_lun = OISST_UNIT

c This is a bit naughty, but I'm going to do it anyway - read the OISST file
c here and close it, it saves passing the lun all over the place and the info
c on whether it is a mandatory input.
                    

c ...    Two different read statements are required for this sequetial
c ...      unformatted file.  One for big endian files where the
c ...      extras bytes used to deliniate different data sections can
c ...      be identified, and one for little endian machines where they
c ...      cannot be identified because the bytes are flipped.  I choose
c ...      to write code to read and throw out all the header bytes
c ...      in this case (10 of them).
         if (big_endian() ) then
            read(oisst_lun) iyrst,imst,idst,iyrnd,imnd,idnd,ndays,
     &                      index
            read(oisst_lun) ((oisst(i,j),i=1,360),j=1,180)
         else
            read(oisst_lun) iyrst,imst,idst,iyrnd,imnd,idnd,ndays,
     &           index,temp1,temp2,((oisst(i,j),i=1,360),j=1,180)
         endif

         do i = 1,360
           do j = 1,180
             if (.not. big_endian() ) then
               oisst(i,j) = sbflip(oisst(i,j))
             end if
           end do
         end do
c        write(*,*)'oisst', oisst(360,180),oisst(359,1),
c    &             oisst(1,100),oisst(90,179),
c    &             oisst(200,90),oisst(300,100)


         goto 180

c ------ Failed to open 
9009     continue 
         call message( 
     &                 FUNCTION,
     &                'FAILED: Could not open OISST file. ',
     &                 0, level 
     &               )

      endif
180   continue
      if( debug.ne.0 ) then
         call strcompress( file, remove_all, len_file )
         write(*,*) '  OISST file: ',file(1:len_file)
      endif

      
C OUTPUT FILES: SST

c --- get the file requirement
      keyname = 'MOD28_REQ'
      keyindex = 1
      status = hdrgetkeystr( filename, keyname, keyindex, req )
      if( status.lt.0 ) req = 'OPTIONAL'
      level = 3
      if( req(1:3).eq.'MAN' ) level = 2

c --- get file access information 
      keyname = 'MOD28_NAME'
      keyindex = 1
      status = hdrgetkeystr(filename, keyname, keyindex,file )
      call strcompress( file, remove_all, len ) 

c --- check for error reading info file
      if( status.lt.0 ) then 
         call message( 
     &                 FUNCTION,
     &                'FAILED: Read of SST output file info . ',
     &                 0, level 
     &               )

      else
                   
c ------ issue the OPEN request
         OPEN( 
     &         FILE=file(1:len),
     &         UNIT=MOD28_UNIT,
     &         STATUS=MOD28_STATUS,
     &         ACCESS=MOD28_ACCESS,
     &         RECL=(max_pixel) * 4,
     &         IOSTAT=io_err, 
     &         ERR=9017
     &       )

         call message(
     &                 '--',
     &                'Creating SST Output file: '// file ,
     &                 0, 4
     &               )

         mod28_lun = MOD28_UNIT
         goto 260

c ------ Failed to open 
9017     continue 
         call message( 
     &                 FUNCTION,
     &                'FAILED: Could not open output file. ',
     &                 0, level 
     &               )

      endif
260   continue
      if( debug.ne.0 ) then
         call strcompress( file, remove_all, len_file )
         write(*,*) '  SST Output file: ',file(1:len_file)
      endif

C OUTPUT FILES: SST HEADER FILE

c --- get the file requirement
      keyname = 'MOD28_REQ'
      keyindex = 1
      status = hdrgetkeystr( filename, keyname, keyindex, req )
      if( status.lt.0 ) req = 'OPTIONAL'
      level = 3
      if( req(1:3).eq.'MAN' ) level = 2

c --- get file access information 
      keyname = 'MOD28_HDR'
      keyindex = 1
      status = hdrgetkeystr( filename, keyname, keyindex,file )
      call strcompress( file, remove_all, len ) 


c --- check for error reading info file
      if( status.lt.0 ) then 
         call message( 
     &                 FUNCTION,
     &                'FAILED: Read of SST output header file info . ',
     &                 0, level 
     &               )

      else
                   
c ------ issue the OPEN request
         OPEN( 
     &         FILE=file(1:len),
     &         UNIT=MOD28_HDR_UNIT,
     &         STATUS=MOD28_HDR_STATUS,
     &         ACCESS=MOD28_HDR_ACCESS,
     &         IOSTAT=io_err, 
     &         ERR=9018
     &       ) 

         hdr_lun = MOD28_HDR_UNIT
         goto 261

c ------ Failed to open 
9018     continue 
         call message( 
     &                 FUNCTION,
     &                'FAILED: Could not open output header file. ',
     &                 0, level 
     &               )

      endif
261   continue
      if( debug.ne.0 ) then
         call strcompress( file, remove_all, len_file )
         write(*,*) '  SST Output header file: ',file(1:len_file)
      endif


C  RUNTIME DEBUG FILE

c --- get the file requirement
      keyname = 'DEBUG_REQ'
      keyindex = 1
      status = hdrgetkeystr( filename, keyname, keyindex, req )
      if( status.lt.0 ) req = 'OPTIONAL'
      level = 3
      if( req(1:3).eq.'MAN' ) level = 2

c --- get file access information 
      keyname = 'DEBUG_NAME'
      keyindex = 1
      status = hdrgetkeystr( filename, keyname, keyindex, file )
      call strcompress( file, remove_all, len ) 

c --- check for error reading info file
      if( status.lt.0 ) then 
         call message( 
     &                 FUNCTION,
     &                'FAILED: Read of RunTime Debug file info. ',
     &                 0, level 
     &               )

      else
                   
c ------ issue the OPEN request
         OPEN( 
     &         FILE=file(1:len),
     &         UNIT=DEBUG_UNIT,
     &         STATUS=DEBUG_STATUS,
     &         FORM=DEBUG_FORM,
     &         ACCESS=DEBUG_ACCESS,
     &         IOSTAT=io_err, 
     &         ERR=9030 
     &       ) 

         debug_lun = DEBUG_UNIT
         goto 270

c ------ Failed to open 
9030     continue 
         call message( 
     &                 FUNCTION,
     &                'FAILED: Open RunTime Debug file. ',
     &                 0, level 
     &               )

      endif
270   continue
      if( debug.ne.0 ) then
         call strcompress( file, remove_all, len_file )
         write(*,*) '  Debug file: ',file(1:len_file)
      endif

      return
      end

c ***************************************************************

        real function sbflip(spin)

c ...   Hal Wolf routine which will flip bytes for a
c ...   real*4 variable (spin)
c ....  version of 10.01.02

        implicit none
        character*1 ch
        character*4 carg
        real spin,spout
        equivalence (carg,spout)

        save

        spout = spin
        ch = carg(1:1)
        carg(1:1) = carg(4:4)
        carg(4:4) = ch
        ch = carg(2:2)
        carg(2:2) = carg(3:3)
        carg(3:3) = ch
        sbflip = spout

        return
        end
